#!/usr/bin/env python
"""Translates text to "kennyspeak" and back.

Idea and implementation of the original *Kenny Translator* by Kohan Ikin:

  http://www.namesuppressed.com/kenny/

Project URL: http://bitbucket.org/blackjack/kennyspeak.py


License
=======

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.
"""
from __future__ import with_statement
import string
import sys
from optparse import OptionParser

__author__ = "Marc 'BlackJack' Rintsch"
__date__ = '2010-07-17'
__version__ = '0.1'
__docformat__ = 'reStructuredText'
__license__ = 'WTFPL'

needs_pass=False
ALPHABET = 'mpf'
ORD_A = ord('a')

def kennify_character(character):
    """Converts given character to kennyspeak.
    
    Alphanumeric letters are converted to three-letter codes.
    Everything else is returned as is.
    """
    if not character.isalpha():
        return character
    value = ord(character.lower()) - ORD_A
    triple = ''.join(ALPHABET[i]
                     for i in [value // 9, value % 9 // 3, value % 9 % 3])
    if character.isupper():
        triple = triple.capitalize()
    return triple


def encode(text):
    """Applies `kennify_character()` to every character in given text.
    
    Example:
    
    >>> kennify('Hello, World!')
    'Mfpmpppmfpmfppf, Fppppfpffpmfmpm!'
    """
    return ''.join(map(kennify_character, text))


def decode(text):
    """Decodes kennyspeak.
    
    >>> unkennify('Mfpmpppmfpmfppf, Fppppfpffpmfmpm!')
    'Hello, World!'
    
    :raises ValueError: if text contains letters outside Kenny's alphabet
        or if there are incomplete letter triples.
    """
    result = list()
    text_iterator = iter(text)
    for character in text_iterator:
        try:
            first = ALPHABET.index(character.lower())
        except ValueError:
            result.append(character)
        else:
            try:
                second = ALPHABET.index(next(text_iterator))
                third = ALPHABET.index(next(text_iterator))
            except (ValueError, StopIteration):
                raise ValueError('text is not kennyspeak')
            decoded = chr(first * 9 + second * 3 + third + ORD_A)
            if character.isupper():
                decoded = decoded.upper()
            result.append(decoded)
    return ''.join(result)
