
#!/usr/bin/env python
"""
Ein Leet-Speak übersetzer.

@author: U{Robert Könitz<mailto:robert.koenitz@googlemail.com>}
@license: U{Gnu General Public License, Version 2, June 1991 \
            <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>}
"""
__version__ = "0.0.7"
__date__ = "2011-01-19"

__author__ = "Robert Könitz"
__contact__ = "robert.koenitz@googlemail.com"

__copyright__ = "Copyright \xa9 Robert Könitz, 2010"
__license__ = "Gnu General Public License, Version 2, June 1991"

__all__ = ["LEET_ENCODE", "LEET_DECODE", "encode", "decode", "gui"]

import random
import select
import sys

needs_pass=False
_with_term = True
_with_gui = True
_leet_length = 5


try:
    import tkinter
except ImportError:
    _with_gui = False

try:
    from argparse import ArgumentParser
except ImportError:
    _with_term = False

LEET_ENCODE = {
    "A" : ("A", "a", "4", "@", "/-\\", "/\\", "^", "ci"),
    "B" : ("B", "b", "8", "|3", "6", "13", "\xc3\x9f", "]3", "I3"),
    "C" : ("C", "c", "(", "<", "\xc2\xa2", "{", "\xc2\xa9"),
    "D" : ("D", "d", "|)", "I)", "|>", "I>", "]>", "[>", "cl", "cI"),
    "E" : ("E", "e", "3", "\xe2\x82\xac", "&amp;", "[-"),
    "F" : ("F", "f", "|=", "I=", "}", "]=", "(=", ")="),
    "G" : ("G", "g", "6", "9", "(_+", "cj"),
    "H" : ("H", "h", "|-|", "#", "]-[", ")-(", "(-)", ":-:", "}{", "}-{", "|-,"),
    "I" : ("I", "i", "1", "!", "|", ";"),
    "J" : ("J", "j", "_|", "_/", "]", "\xc2\xbf", "</", "_)"),
    "K" : ("K", "k", "X", "|<", "I<", "]<", "|X", "|{"),
    "L" : ("L", "l", "|", "|_", "\xc2\xa3", "1", "7", "\xc2\xac"),
    "M" : ("M", "m", "|V|", "|v|", "/\\/\\", "|\\/|", "^^", "nn", "(V)", "(v)",
           "(\\/)", "/|\\", "/|/|", ".\\\\", "/^^\\", "/V\\", "|^^|", ",-,-,"),
    "N" : ("N", "n", "|\|", "/|/", "]\\[", "//", ",-,"),
    "O" : ("O", "o", "0", "()", "[]", "\xc2\xa4", "<>", "@"),
    "P" : ("P", "p", "|o", "|*", "\xc2\xb0", "|>", "|?", "|7", "\xc3\xbe",
           "\xc2\xb6"),
    "Q" : ("Q", "q", "p", "O_", "0,", "(,)", "9", "O,"),
    "R" : ("R", "r", "|2", "/2", "I2", "I^", "|^", "lz", "Iz", "|z", "l2"),
    "S" : ("S", "s", "Z", "z", "5", "$", "\xc2\xa7"),
    "T" : ("T", "t", "7", "+", "-|-", "1", "\xe2\x80\xa0"),
    "U" : ("U", "u", "|_|", "(_)", "\\_/", "\\_\\", "/_/", "]_[", "\xc2\xb5"),
    "V" : ("V", "v", "\\/"),
    "W" : ("W", "w", "\\/\\/", "vv", "VV", "uu", "UU", "'//", "\\\\'", "\\^/",
           "(n)", "\\X/", "\\|/", "\\_|_/", "\\_:_/"),
    "X" : ("X", "x", "><", "}{", "*", ")(", "\xc3\x97"),
    "Y" : ("Y", "y", "\xc2\xa5", "j", "`/", "\xc3\xbf"),
    "Z" : ("Z", "z", "2", "~/_", "7_", "%")
    }

LEET_DECODE = dict()
for key, values in LEET_ENCODE.items():
    for value in values:
        LEET_DECODE[value] = key

def encode(text):
    """Kodiert den übergeben Text nach Leet.

    @type text: C{str}
    @param text: Der Text, welcher übersetzt werden soll.

    @returntype: C{str}
    @return: Den Leet-Text.
    """
    return "".join(random.choice(LEET_ENCODE.get(c, c)) for c in \
                   str(text).upper())

def decode(leet):
    """Dekodiert einen Text von Leet, in einen normalen Text.

    @type leet: C{str}
    @param leet: Der Leet-Text, welcher wieder in einen normalen Text
                 umgewandelt werden soll.

    @returntype: C{str}
    @return: Einen normalen Text.

    @warning: Der Text kann meist nicht vollständig zurück gewandelt
              werden.
    """
    text = []
    index = 0
    while index < len(leet):
        for chunksize in range(_leet_length, 0, -1):
            chars = leet[index:index + chunksize]
            if chars in LEET_DECODE:
                text.append(LEET_DECODE[chars])
                break
            elif chunksize == 1:
                text.append(chars)
        index += chunksize
    return "".join(text)

def gui(leet_text="", normal_text="", switched=False):
    """Eine kleine GUI für den Leet-übersetzer.

    @type leet_text: C{str}
    @param leet_text: Startwert für das Leet-Text-Widget.
   
    @type normal_text: C{str}
    @param normal_text: Startwert für das Normale-Text-Widget.

    @type switched: C{bool}
    @param switched: Zeigt an ob leet- und normal-Text vertauscht sind.
    """
    if switched: leet_text, normal_text = normal_text, leet_text
    pack_style = dict(expand=True, fill="both", padx=2)
    root = tkinter.Tk()
    root.title("Leet \xdcbersetzer")
   
    text = tkinter.Frame(root)
    text.pack(pack_style)

    tkinter.Label(text, text="Normaler Text:", anchor="w").pack(pack_style)
    decode_text = tkinter.Text(text, width=50, height=5)
    decode_text.insert("0.0", leet_text)
    decode_text.pack(pack_style)

    decode_buttons = tkinter.Frame(text)
    decode_buttons.pack(pack_style)
    decode_open = tkinter.Button(decode_buttons, text="\xd6ffnen")
    decode_open.pack(pack_style, side="left")
    decode_save = tkinter.Button(decode_buttons, text="Speichern")
    decode_save.pack(pack_style, side="right")
   
    tkinter.Label(text, text="1337 t&amp;><+:", anchor="w").pack(pack_style)
    encode_text = tkinter.Text(text, width=50, height=5)
    encode_text.insert("0.0", normal_text)
    encode_text.pack(pack_style)

    encode_buttons = tkinter.Frame(text)
    encode_buttons.pack(pack_style)
    encode_open = tkinter.Button(encode_buttons, text="\xd6ffnen")
    encode_open.pack(pack_style, side="left")
    encode_save = tkinter.Button(encode_buttons, text="Speichern")
    encode_save.pack(pack_style, side="right")

    options = tkinter.Frame(root)
    options.pack(pack_style, side="left")
   
    code = tkinter.BooleanVar()
    encode_option = tkinter.Radiobutton(options, text="Kodieren",
                                        variable=code, value=False, anchor="w")
    encode_option.pack(pack_style)
    decode_option = tkinter.Radiobutton(options, text="Dekodieren",
                                        variable=code, value=True, anchor="w")
    decode_option.pack(pack_style)

    def cmd():
        if code.get():
            text = decode(encode_text.get("0.0", "end"))
            decode_text.delete("0.0", "end")
            decode_text.insert("end", text)
        else:
            text = encode(decode_text.get("0.0", "end"))
            encode_text.delete("0.0", "end")
            encode_text.insert("end", text)

       
    translate = tkinter.Button(root, text="übersetzen", command=cmd)
    translate.pack(pack_style)
   
    root.mainloop()

def _parser():
    """Erstellt den ArgumentParser.

    @returntype C{argparse.Namespace}
    @return: Das Ergebnis des ArgumentParser.
    """    
    if not _with_term:
        return None
   
    parser = ArgumentParser(prog="leet.py",
                            description="Ein leet \xdcbersetzer.")
    parser.add_argument("-d", "--decode", dest="decode", action="store_true",
                        help="Dekodiert einen Text")
    parser.add_argument("-w", "--without", dest="without", action="store_true",
                        help="Ohne Ausgabe in der Shell.")
    if _with_gui:
        parser.add_argument("-g", "--gui", dest="gui", action="store_true",
                            help="Erzeugt eine Grafische Bedienoberfläche.")
    parser.add_argument("-o", "--output", type=str, dest="filepath",
                        action="store", help="Gibt an wo der übersetzte " \
                        "Text peichert werden soll.")
    parser.add_argument("--version", action="version",
                        help="Zeigt die Version des Scriptes an.",
                        version="{0} {1}".format(parser.prog, __version__))
    return parser.parse_args()
           
if __name__ == "__main__":
    result = _parser()
    if result is not None:
        if select.select([sys.stdin, ],[],[], 0.0)[0]:
            text = sys.stdin.read()
            if not result.without and not result.gui:
                print("Text: {0}".format(text))
        else:
            text = input("Text: ")
           
        code = decode(text) if result.decode else encode(text)
        if not result.without and not result.gui:
            print("übersetzung: {0}".format(code))

        if result.filepath is not None:
            with open(result.filepath, "w") as handle:
                handle.write(code)
       
        if result.gui and _with_gui:
            gui(text, code, result.decode)
    else:
        gui()
